require 'rexml/document'
require 'facter'

module Puppet_X
    module EnterpriseModules
      module Oracle
        module Facts

          def get_database_user(home)
            databaseUser = Facter.value('override_database_user')
            if databaseUser.nil?
              # puts "database user is oracle"
              if File.exist?("#{home}/bin")
                Facter::Util::Resolution.exec("/bin/stat --format=%U #{home}/bin")
              else
                'oracle'
              end
            else
              # puts "database user is " + databaseUser
              return databaseUser
            end
          end

          def get_su_command
            os = Facter.value(:kernel)
            if os == 'Linux'
              return 'su -l '
            elsif os == 'SunOS'
              return 'su - '
            end
            'su -l '
          end

          def get_ora_inv_path
            os = Facter.value(:kernel)
            if 'Linux' == os
              return '/etc'
            elsif 'SunOS' == os
              return '/var/opt/oracle'
            end
            '/etc'
          end

          def get_opatch_version(name)
            if File.exist?(name + '/OPatch/opatch')
              opatchOut = Facter::Util::Resolution.exec(name + '/OPatch/opatch version')
            else
              opatchOut = ''
            end
            if opatchOut.nil?
              opatchver = 'Error;'
            else
              opatchver = opatchOut.split(' ')[2]
            end
            Puppet.debug "ora_install opatch version #{name}/OPatch/opatch: #{opatchver}"
            opatchver
          end

          def get_orainst_loc
            if File.exist?(get_ora_inv_path + '/oraInst.loc')
              str = ''
              output = File.read(get_ora_inv_path + '/oraInst.loc')
              output.split(/\r?\n/).each do |item|
                if item.match(/^inventory_loc/)
                  str = item[14, 50]
                end
              end
              return str
            else
              return ''
            end
          end

          def get_homes(path)
            unless path.nil?
              if File.exist?(path + '/ContentsXML/inventory.xml')
                file = File.read(path + '/ContentsXML/inventory.xml')
                doc = REXML::Document.new(file)
                homes = []
                doc.elements.each('/INVENTORY/HOME_LIST/HOME') do |element|
                  str = element.attributes['LOC']
                  unless str.nil?
                    if str.include? 'plugins'
                      # skip EM agent
                    elsif str.include? 'jdk'
                      # skip EM agent
                    elsif str.include? 'OraPlaceHolderDummyHome'
                      # skip EM agent
                    else
                      homes << str
                    end
                  end
                end
                return homes
              else
                return {}
              end
            else
              return {}
            end
          end

          def get_version(home)
            opatch_installed?(home) ? Facter::Util::Resolution.exec("su #{get_database_user(home)} #{home}/OPatch/opatch lsinventory | awk '/^(Oracle Database|Oracle Client|Oracle Grid Infrastructure|Oracle GoldenGate)/ {print $NF}'") : ''
          end

          def get_installed_patches(oracle_home)
            ora_user = user_for_file(oracle_home)
            patches_in_home(oracle_home, ora_user).flatten.sort
          end

          def patches_in_home(oracle_product_home_dir, os_user)
            if opatch_installed?(oracle_product_home_dir)
              full_command = "#{oracle_product_home_dir}/OPatch/opatch lsinventory -oh #{oracle_product_home_dir}"
              raw_list = execute(full_command, os_user)
              raw_list.scan(/Patch\s.(\d+)\s.*:\sapplied on/).flatten
              # patch_ids.collect { |p| "#{oracle_product_home_dir}:#{p}" }
            else
              #
              # When Opatch is not installed, we don't report any patches
              #
              []
            end
          end

          def opatch_installed?(oracle_home)
            File.exist?("#{oracle_home}/OPatch/opatch")
          end

          def user_for_file(home)
            #
            # To run opatch, we need the owner of the grid and oracle directory. The mechanism
            # we use to get this user, is to fetch the owner of the bin directory in the grid or
            # oracle home. Sometimes however, these directories also contain files with root as owner.
            # since root can never be the user of the grid or oracle processen, we skip those.
            #
            a_file = first_non_root_file(home)
            uid = File.stat("#{home}/#{a_file}").uid
            Etc.getpwuid(uid).name
          end

          def first_non_root_file(dir)
            Dir.new(dir).each { |file| break file if File.stat("#{dir}/#{file}").uid.nonzero? }
          end

          def execute(command, user)
            full_command = if user
                             "su - #{user} -c \"#{command}\""
                           else
                             command
                           end
            `#{full_command}`
          end

          def homes_hash
            homes = get_homes(get_orainst_loc)
            homes_array = homes.collect do |home|
              [home, { 'product_version'   => get_version(home),
                       'opatch_version'    => get_opatch_version(home),
                       'installed_patches' => get_installed_patches(home) }]
            end
            Hash[homes_array]
          end

        end
      end
    end
  end
