#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: ora_install::client
#
#
define ora_install::client(
  # lint:ignore:2sp_soft_tabs
  Enum['11.2.0.1','11.2.0.4','12.1.0.1','12.1.0.2','12.2.0.1']
                                      $version                   = undef,
  String[1]                           $file                      = undef,
  Stdlib::Absolutepath                $oracle_base               = undef,
  Stdlib::Absolutepath                $oracle_home               = undef,
  Integer                             $db_port                   = 1521,
  String[1]                           $user                      = 'oracle',
  Stdlib::Absolutepath                $user_base_dir             = '/home',
  String[1]                           $group                     = 'dba',
  String[1]                           $group_install             = 'oinstall',
  Stdlib::Absolutepath                $download_dir              = '/install',
  Stdlib::Absolutepath                $temp_dir                  = '/tmp',
  Boolean                             $bash_profile              = true,
  Optional[String[1]]                 $puppet_download_mnt_point = undef,
  Optional[Boolean]                   $remote_file               = undef,
  Variant[Boolean,Enum['on_failure']] $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
  # lint:endignore
)
{
  ora_install { $title: ensure => 'present'}
  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  $ora_inventory = "${oracle_base}/oraInventory"

  db_directory_structure { "client structure ${version}":
    ensure            => present,
    oracle_base_dir   => $oracle_base,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    os_user           => $user,
    os_group          => $group_install,
  }

  unless ( oracle_exists( $oracle_home ) ) {

    notify { "ora_install::installdb ${oracle_home} does not exists": }

    $exec_path     = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

    if ( $puppet_download_mnt_point ) {
      $mount_point     = $puppet_download_mnt_point
    } else {
      $mount_point     = 'puppet:///modules/ora_install/'
    }

    if ( $mount_point == $download_dir ) {
      $archive_cleanup = false
    } else {
      $archive_cleanup = true
    }

    file { "${download_dir}/client_${version}":
      ensure => directory,
      owner  => $user,
      group  => $group,
      mode   => '0775',
    }

    archive { "${download_dir}/${file}":
      ensure       => present,
      cleanup      => $archive_cleanup,
      creates      => "${download_dir}/client_${version}/client",
      extract      => true,
      extract_path => "${download_dir}/client_${version}",
      group        => $group,
      require      => File["${download_dir}/client_${version}"],
      source       => "${mount_point}/${file}",
      temp_dir     => $temp_dir,
      user         => $user,
    }

    ora_install::utils::dborainst{"oracle orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group_install,
    }

    unless ( defined(File["${download_dir}/db_client_${version}.rsp"]) ) {
      file { "${download_dir}/db_client_${version}.rsp":
        ensure  => present,
        content => template("ora_install/db_client_${version}.rsp.erb"),
        mode    => '0775',
        owner   => $user,
        group   => $group,
        require => [Ora_install::Utils::Dborainst["oracle orainst ${version}"],
                    Db_directory_structure["client structure ${version}"],],
      }
    }

    exec { "install oracle client ${title}":
      command     => "/bin/sh -c 'unset DISPLAY;${download_dir}/client_${version}/client/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/db_client_${version}.rsp'",
      require     => [Ora_install::Utils::Dborainst["oracle orainst ${version}"],
                      File["${download_dir}/db_client_${version}.rsp"],
                      Archive["${download_dir}/${file}"]],
      creates     => $oracle_home,
      environment => ["TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
      timeout     => 0,
      returns     => [6,0],
      path        => $exec_path,
      user        => $user,
      group       => $group_install,
      logoutput   => $logoutput,
    }

    exec { "run root.sh script ${title}":
      command   => "${oracle_home}/root.sh",
      user      => 'root',
      group     => 'root',
      require   => Exec["install oracle client ${title}"],
      path      => $exec_path,
      logoutput => $logoutput,
    }

    file { "${download_dir}/netca_client_${version}.rsp":
      ensure  => present,
      content => template("ora_install/netca_client_${version}.rsp.erb"),
      require => Exec["run root.sh script ${title}"],
      mode    => '0775',
      owner   => $user,
      group   => $group,
    }

    exec { "install oracle net ${title}":
      command   => "${oracle_home}/bin/netca /silent /responsefile ${download_dir}/netca_client_${version}.rsp",
      require   => [File["${download_dir}/netca_client_${version}.rsp"],Exec["run root.sh script ${title}"],],
      creates   => "${oracle_home}/network/admin/sqlnet.ora",
      path      => $exec_path,
      user      => $user,
      group     => $group,
      logoutput => $logoutput,
    }

    if ( $bash_profile ) {
      unless ( defined(File["${user_base_dir}/${user}/.bash_profile"]) ) {
        file { "${user_base_dir}/${user}/.bash_profile":
          ensure  => present,
          content => regsubst(template('ora_install/bash_profile.erb'), '\r\n', "\n", 'EMG'),
          mode    => '0775',
          owner   => $user,
          group   => $group,
        }
      }
    }

    # cleanup
    exec { "remove oracle client extract folder ${title}":
      command => "rm -rf ${download_dir}/client_${version}",
      user    => 'root',
      group   => 'root',
      path    => $exec_path,
      require => Exec["install oracle net ${title}"],
    }
    if ( $archive_cleanup ) {
      file { "${download_dir}/${file}":
        ensure => absent,
      }
    }
  }
}
