#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: ora_install::database
#
define ora_install::database(
  # lint:ignore:2sp_soft_tabs
  String[1]                           $oracle_base               = undef,
  String[1]                           $oracle_home               = undef,
  Enum['11.2','12.1','12.2']          $version                   = '12.2',
  String[1]                           $user                      = 'oracle',
  String[1]                           $group                     = 'dba',
  Stdlib::Absolutepath                $download_dir              = '/install',
  Enum['create','delete']             $action                    = 'create',
  Optional[String[1]]                 $template                  = undef,
  Optional[Enum['non-seed','seed']]   $template_type             = 'non-seed',
  String[1]                           $db_name                   = 'orcl',
  Optional[String[1]]                 $db_domain                 = undef,
  Optional[Integer]                   $db_port                   = 1521,
  String[1]                           $sys_password              = 'Welcome01!',
  String[1]                           $system_password           = 'Welcome01!',
  Optional[String[1]]                 $data_file_destination     = undef,
  Optional[String[1]]                 $recovery_area_destination = undef,
  String[1]                           $character_set             = 'AL32UTF8',
  String[1]                           $nationalcharacter_set     = 'AL16UTF16',
  Optional[Variant[String[1], Hash]]  $init_params               = undef,
  Enum['TRUE','FALSE']                $sample_schema             = 'TRUE',
  Integer                             $memory_percentage         = 40,
  Integer                             $memory_total              = 1000,
  Optional[Enum['AUTO','AUTO_SGA','CUSTOM_SGA']]
                                      $memory_mgmt_type          = 'AUTO_SGA',
  Optional[Enum['MULTIPURPOSE','DATA_WAREHOUSING','OLTP']]
                                      $database_type             = 'MULTIPURPOSE',
  Enum['CENTRAL','LOCAL','ALL','NONE','DBEXPRESS','BOTH']
                                      $em_configuration          = 'NONE',
  Enum['FS','CFS','ASM']              $storage_type              = 'FS',
  String[1]                           $asm_snmp_password         = 'Welcome01!',
  String[1]                           $db_snmp_password          = 'Welcome01!',
  Optional[String[1]]                 $asm_diskgroup             = 'DATA',
  Optional[String[1]]                 $recovery_diskgroup        = undef,
  Optional[String[1]]                 $cluster_nodes             = undef,
  Boolean                             $container_database        = false,
  Optional[String[1]]                 $puppet_download_mnt_point = undef,
  Variant[Boolean,Enum['on_failure']] $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
  # lint:endignore
)
{
  ora_install { $title: ensure => 'present'}

  $sanitized_title = regsubst($title, '[^a-zA-Z0-9.-]', '_', 'G')

  if ( $action == 'create' ) {
    $operation_type = 'createDatabase'
  } elsif ( $action == 'delete' ) {
    $operation_type = 'deleteDatabase'
  }

  case $version {
    '11.2': {
      unless ( $em_configuration in ['CENTRAL','LOCAL','ALL','NONE'] ) { fail("em_configuration should be defined as CENTRAL, LOCAL, ALL or NONE for version ${version}") }
      if ( $container_database ) { fail('container or pluggable database is not supported on version 11.2') }
      $action_param = ''
      $delete_command = "${oracle_home}/bin/dbca -silent ${action_param} -responseFile ${download_dir}/database_${sanitized_title}.rsp"
      $datafile_dest_param = ''
      $memory_mgmt_type_param = ''
    }
    '12.1': {
      unless ( $em_configuration in ['CENTRAL','LOCAL','ALL','NONE'] ) { fail("em_configuration should be defined as CENTRAL, LOCAL, ALL or NONE for version ${version}") }
      $action_param = ''
      $delete_command = "${oracle_home}/bin/dbca -silent ${action_param} -responseFile ${download_dir}/database_${sanitized_title}.rsp"
      $datafile_dest_param = ''
      $memory_mgmt_type_param = ''
    }
    '12.2': {
      unless ( $em_configuration in ['CENTRAL','DBEXPRESS','BOTH','NONE'] ) { fail("em_configuration should be defined as CENTRAL, DBEXPRESS, BOTH or NONE for version ${version}") }
      $action_param = "-${operation_type}"
      # TODO: Check this for RAC, -sid should be different
      $delete_command = "${oracle_home}/bin/dbca -silent ${action_param} -sourceDB ${sanitized_title} -sid ${sanitized_title} -sysDBAPassword \"${sys_password}\" -sysDBAUserName sys"
      $datafile_dest_param = "-datafileDestination ${data_file_destination}"
      $memory_mgmt_type_param = "-memoryMgmtType ${memory_mgmt_type}"
    }
    default: {}
  }

  # if ( $memory_percentage and $memory_total ) { fail("cannot specify both memory_percentage and memory_total, choose one") }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  if ( $puppet_download_mnt_point ) {
    $mount_point = $puppet_download_mnt_point
  } else {
    $mount_point = 'ora_install/'
  }

  case $::kernel {
    'Linux': {
      $user_home = "/home/${user}"
    }
    'SunOS': {
      $user_home = "/export/home/${user}"
    }
    default: {
      fail('Unrecognized operating system')
    }
  }

  if ( $init_params =~ Hash or $init_params =~ String ) {
    if ( $init_params =~ Hash ) {
      $init_params_array = sort(join_keys_to_values($init_params, '='))
      $sanitized_ini_params = join($init_params_array,',')
    } else {
      $sanitized_ini_params = $init_params
    }
  }

  if ( $db_domain ) {
    $globaldb_name = "${db_name}.${db_domain}"
  } else {
    $globaldb_name = $db_name
  }

  unless ( defined(File["${download_dir}/database_${sanitized_title}.rsp"]) ) {
    file { "${download_dir}/database_${sanitized_title}.rsp":
      ensure  => present,
      content => template("ora_install/dbca_${version}.rsp.erb"),
      mode    => '0770',
      owner   => $user,
      group   => $group,
      before  => Exec["oracle database ${title}"],
    }
  }

  if ( $template ) {
    $template_extention = $template_type ? {
      'non-seed' => 'dbt',
      'seed'     => 'dbc',
    }

    $templatename = "${download_dir}/${template}_${sanitized_title}.${template_extention}"
    file { $templatename:
      ensure  => present,
      content => template("${mount_point}/${template}.${template_extention}.erb"),
      mode    => '0775',
      owner   => $user,
      group   => $group,
      before  => Exec["oracle database ${title}"],
    }
  }

  if ( $action == 'create' ) {
    if ( $template ) {
      unless ( $container_database ) {
        $command = "${oracle_home}/bin/dbca -silent -createDatabase -templateName ${templatename} -gdbname ${globaldb_name} -responseFile NO_VALUE -sysPassword ${sys_password} -systemPassword ${system_password} -dbsnmpPassword ${db_snmp_password} -asmsnmpPassword ${asm_snmp_password} -storageType ${storage_type} ${datafile_dest_param} -emConfiguration ${em_configuration} -characterSet ${character_set} -nationalCharacterSet ${nationalcharacter_set} ${memory_mgmt_type_param}"
      } else {
        $command = "${oracle_home}/bin/dbca -silent -createDatabase -templateName ${templatename} -gdbname ${globaldb_name} -createAsContainerDatabase ${container_database} -responseFile NO_VALUE -sysPassword ${sys_password} -systemPassword ${system_password} -dbsnmpPassword ${db_snmp_password} -asmsnmpPassword ${asm_snmp_password} -storageType ${storage_type} ${datafile_dest_param} -emConfiguration ${em_configuration} -characterSet ${character_set} -nationalCharacterSet ${nationalcharacter_set} ${memory_mgmt_type_param}"
      }
    } else {
      $command = "${oracle_home}/bin/dbca -silent ${action_param} -responseFile ${download_dir}/database_${sanitized_title}.rsp"
    }
    exec { "oracle database ${title}":
      command     => $command,
      creates     => "${oracle_base}/admin/${db_name}",
      timeout     => 0,
      path        => $exec_path,
      user        => $user,
      group       => $group,
      cwd         => $oracle_base,
      environment => ["USER=${user}",],
      logoutput   => $logoutput,
    }
  } elsif ( $action == 'delete' ) {
    exec { "oracle database ${title}":
      command     => $delete_command,
      onlyif      => "ls ${oracle_base}/admin/${db_name}",
      timeout     => 0,
      path        => $exec_path,
      user        => $user,
      group       => $group,
      cwd         => $oracle_base,
      environment => ["USER=${user}",],
      logoutput   => $logoutput,
    }
  }
}
