#
# See the file "LICENSE" for the full license governing this code.
#
# == Defined type : ora_install::installdb
#
# See the documentation folder for documentation
#
define ora_install::installdb(
  # lint:ignore:2sp_soft_tabs
  Enum['12.2.0.1','12.1.0.1','12.1.0.2','11.2.0.3','11.2.0.4', '11.2.0.1']
                                               $version                   = undef,
  Stdlib::Absolutepath                         $oracle_base               = undef,
  Stdlib::Absolutepath                         $oracle_home               = undef,
  String[1]                                    $file                      = undef,
  Enum['SE2', 'SE', 'EE', 'SEONE']             $database_type             = 'SE2',
  Optional[Stdlib::Absolutepath]               $ora_inventory_dir         = undef,
  Boolean                                      $ee_options_selection      = false,
  Optional[String[1]]                          $ee_optional_components    = undef, # 'oracle.rdbms.partitioning:11.2.0.4.0,oracle.oraolap:11.2.0.4.0,oracle.rdbms.dm:11.2.0.4.0,oracle.rdbms.dv:11.2.0.4.0,oracle.rdbms.lbac:11.2.0.4.0,oracle.rdbms.rat:11.2.0.4.0'
  Boolean                                      $bash_profile              = true,
  String[1]                                    $user                      = 'oracle',
  Stdlib::Absolutepath                         $user_base_dir             = '/home',
  String[1]                                    $group                     = 'dba',
  String[1]                                    $group_install             = 'oinstall',
  String[1]                                    $group_oper                = 'oper',
  Stdlib::Absolutepath                         $download_dir              = '/install',
  Stdlib::Absolutepath                         $temp_dir                  = '/tmp',
  Boolean                                      $zip_extract               = true,
  Optional[String[1]]                          $puppet_download_mnt_point = undef,
  Optional[Boolean]                            $remote_file               = undef,
  Optional[String[1]]                          $cluster_nodes             = undef,
  Boolean                                      $cleanup_install_files     = true,
  Boolean                                      $is_rack_one_install       = false,
  Variant[Boolean, Enum['on_failure']]         $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
  # lint:endignore
)
{
  ora_install { $title: ensure => 'present'}

  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  unless ( $::kernel in ['Linux','SunOS'] ) {
    fail('Unrecognized operating system, please use it on a Linux or SunOS host')
  }

  unless ( $oracle_base in $oracle_home ){
    fail('oracle_home folder should be under the oracle_base folder')
  }

  case $version {
    '11.2.0.1', '12.1.0.1': {
      unless ( $database_type in ['EE', 'SE', 'SEONE'] ) {
        fail("Unrecognized database_type(${database_type}) use EE, SE or SEONE for version ${version}")
      }
      $files = ["${file}_1of2.zip", "${file}_2of2.zip"]
    }
    '11.2.0.3', '11.2.0.4': {
      unless $database_type in ['EE', 'SE', 'SEONE'] {
        fail("Unrecognized database_type(${database_type}) use EE, SE or SEONE for version ${version}")
      }
      $files =  ["${file}_1of7.zip", "${file}_2of7.zip"]
      }
    '12.1.0.2': {
      unless $database_type in ['EE', 'SE'] {
        fail("Unrecognized database_type(${database_type}) use EE or SE2 for version ${version}")
      }
      $files = ["${file}_1of2.zip", "${file}_2of2.zip"]
    }
    '12.2.0.1': {
      unless $database_type in ['EE', 'SE2'] {
        fail("Unrecognized database_type(${database_type}) use EE or SE2 for version ${version}")
      }
      $files = ["${file}.zip"]
    }
    default: {}
  }

  $exec_path     = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  if ( $puppet_download_mnt_point ) {
    $mount_point     = $puppet_download_mnt_point
  } else {
    $mount_point     = 'puppet:///modules/ora_install/'
  }

  if ( $mount_point == $download_dir ) {
    $archive_cleanup = false
  } else {
    $archive_cleanup = true
  }

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${oracle_base}/oraInventory"
  }

  unless ( oracle_exists( $oracle_home ) ) {

    db_directory_structure{ "oracle structure ${version}":
      ensure            => present,
      oracle_base_dir   => $oracle_base,
      ora_inventory_dir => $ora_inventory,
      download_dir      => $download_dir,
      os_user           => $user,
      os_group          => $group_install,
    }

    if ( $zip_extract ) {

      file { "${download_dir}/${file}":
        ensure => directory,
        owner  => $user,
        group  => $group,
        mode   => '0755',
      }

      $files.each | $filename | {
        archive { "${download_dir}/${filename}":
          ensure       => present,
          before       => Exec["install oracle database ${title}"],
          cleanup      => $archive_cleanup,
          creates      => "${download_dir}/${file}/installdb",
          group        => $group_install,
          extract      => true,
          extract_path => "${download_dir}/${file}",
          require      => Db_directory_structure["oracle structure ${version}"],
          source       => "${mount_point}/${filename}",
          temp_dir     => $temp_dir,
          user         => $user,
        }
      }
    }

    ora_install::utils::dborainst{"database orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group_install,
    }

    unless ( defined(File["${download_dir}/db_install_${version}.rsp"]) ) {
      file { "${download_dir}/db_install_${version}.rsp":
        ensure  => present,
        content => template("ora_install/db_install_${version}.rsp.erb"),
        mode    => '0775',
        owner   => $user,
        group   => $group,
        require => [Ora_install::Utils::Dborainst["database orainst ${version}"],
                    Db_directory_structure["oracle structure ${version}"]],
      }
    }

    exec { "install oracle database ${title}":
      command     => "/bin/sh -c 'unset DISPLAY;${download_dir}/${file}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/db_install_${version}.rsp'",
      creates     => "${oracle_home}/dbs",
      environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
      timeout     => 0,
      returns     => [6,0],
      path        => $exec_path,
      user        => $user,
      group       => $group_install,
      cwd         => $oracle_base,
      logoutput   => $logoutput,
      require     => [Ora_install::Utils::Dborainst["database orainst ${version}"],
                      File["${download_dir}/db_install_${version}.rsp"]],
    }

    if ( $bash_profile ) {
      unless ( defined(File["${user_base_dir}/${user}/.bash_profile"]) ) {
        file { "${user_base_dir}/${user}/.bash_profile":
          ensure  => present,
          content => regsubst(template('ora_install/bash_profile.erb'), '\r\n', "\n", 'EMG'),
          mode    => '0775',
          owner   => $user,
          group   => $group,
        }
      }
    }

    exec { "run root.sh script ${title}":
      command   => "${oracle_home}/root.sh",
      user      => 'root',
      group     => 'root',
      path      => $exec_path,
      cwd       => $oracle_base,
      logoutput => $logoutput,
      require   => Exec["install oracle database ${title}"],
    }

    file { $oracle_home:
      ensure  => directory,
      recurse => false,
      replace => false,
      mode    => '0775',
      owner   => $user,
      group   => $group_install,
      require => Exec["install oracle database ${title}","run root.sh script ${title}"],
    }

    # cleanup
    if ( $cleanup_install_files ) {
      if ( $zip_extract ) {
        exec { "remove oracle db extract folder ${title}":
          command   => "rm -rf ${download_dir}/${file}",
          user      => 'root',
          group     => 'root',
          path      => $exec_path,
          cwd       => $oracle_base,
          logoutput => $logoutput,
          require   => [Exec["install oracle database ${title}"],
                        Exec["run root.sh script ${title}"],],
        }
      }
      if ( $archive_cleanup ) {
        $files.each | $filename | {
          file { "${download_dir}/${filename}":
            ensure => absent,
          }
        }
      }
    }
  }
}
