#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: ora_install::installem
#
#
define ora_install::installem(
  Enum['12.1.0.4','12.1.0.5','13.1.0.0','13.2.0.0'] $version                       = '12.1.0.5',
  String[1]                                         $file                          = undef,
  Optional[Stdlib::Absolutepath]                    $ora_inventory_dir             = undef,
  Stdlib::Absolutepath                              $oracle_base_dir               = undef,
  Stdlib::Absolutepath                              $oracle_home_dir               = undef,
  Stdlib::Absolutepath                              $agent_base_dir                = undef,
  Stdlib::Absolutepath                              $software_library_dir          = undef,
  String[1]                                         $weblogic_user                 = 'weblogic',
  String[1]                                         $weblogic_password             = undef,
  String[1]                                         $database_hostname             = undef,
  Integer                                           $database_listener_port        = 1521,
  String[1]                                         $database_service_sid_name     = undef,
  String[1]                                         $database_sys_password         = undef,
  String[1]                                         $sysman_password               = undef,
  String[1]                                         $agent_registration_password   = undef,
  Enum['SMALL','MEDIUM','LARGE']                    $deployment_size               = 'SMALL',
  String[1]                                         $user                          = 'oracle',
  String[1]                                         $group                         = 'oinstall',
  Stdlib::Absolutepath                              $download_dir                  = '/install',
  Stdlib::Absolutepath                              $temp_dir                      = '/tmp',
  Boolean                                           $zip_extract                   = true,
  String[1]                                         $puppet_download_mnt_point     = undef,
  Optional[Boolean]                                 $remote_file                   = undef,
  Integer                                           $admin_server_https_port       = 7101,
  Integer                                           $managed_server_http_port      = 7201,
  Integer                                           $managed_server_https_port     = 7301,
  Integer                                           $em_upload_http_port           = 4889,
  Integer                                           $em_upload_https_port          = 1159,
  Integer                                           $em_central_console_http_port  = 7788,
  Integer                                           $em_central_console_https_port = 7799,
  Integer                                           $bi_publisher_http_port        = 9701,
  Integer                                           $bi_publisher_https_port       = 9801,
  Integer                                           $nodemanager_https_port        = 7401,
  Integer                                           $agent_port                    = 3872,
  Optional[Variant[Boolean,Enum['on_failure']]]     $log_output                    = undef,
  Variant[Boolean,Enum['on_failure']]               $logoutput                     = lookup({name => 'logoutput', default_value => 'on_failure'}),
)
{
  ora_install { $title: ensure => 'present'}

  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  unless ( $::kernel in ['Linux','SunOS'] ) {
    fail('Unrecognized operating system, please use it on a Linux or SunOS host')
  }

  case $version {
    '12.1.0.4','12.1.0.5': {
      $em_files = ["${file}_disk1.zip", "${file}_disk2.zip", "${file}_disk3.zip"]
      $extract_files = true
      $install_command = "/bin/su - ${user} -c 'unset DISPLAY;${download_dir}/${file}/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/em_install_${version}.rsp'"
      $rootsh = "${oracle_home_dir}/oms/allroot.sh"
    }
    '13.1.0.0': {
      $em_files = ["${file}.bin", "${file}-2.zip", "${file}-3.zip", "${file}-4.zip", "${file}-5.zip"]
      $extract_files = false
      $install_command = "/bin/su - ${user} -c 'unset DISPLAY;${download_dir}/${file}.bin -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/em_install_${version}.rsp -J-Djava.io.tmpdir=${temp_dir}'"
      $rootsh = "${oracle_home_dir}/allroot.sh"
    }
    '13.2.0.0': {
      if ( $file =~ /em13200p1/ ) {
        $em_files = ["${file}.bin", "${file}-2.zip", "${file}-3.zip", "${file}-4.zip", "${file}-5.zip", "${file}-6.zip", "${file}-7.zip"]
      } else {
        $em_files = ["${file}.bin", "${file}-2.zip", "${file}-3.zip", "${file}-4.zip", "${file}-5.zip", "${file}-6.zip"]
      }
      $extract_files = false
      $install_command = "/bin/su - ${user} -c 'unset DISPLAY;${download_dir}/${file}.bin -silent -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/em_install_${version}.rsp -J-Djava.io.tmpdir=${temp_dir}'"
      $rootsh = "${oracle_home_dir}/allroot.sh"
    }
    default: {}
  }

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${oracle_base_dir}/oraInventory"
  }

  db_directory_structure{"oracle em structure ${version}":
    ensure            => present,
    oracle_base_dir   => $oracle_base_dir,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    os_user           => $user,
    os_group          => $group,
  }

  unless ( oracle_exists( $oracle_home_dir ) ) {

    notify { "ora_install::installem ${oracle_home_dir} does not exists": }

    $exec_path     = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

    if ( $puppet_download_mnt_point ) {
      $mount_point     = $puppet_download_mnt_point
    } else {
      $mount_point     = 'puppet:///modules/ora_install/'
    }

    if ( $mount_point == $download_dir ) {
      $archive_cleanup = false
    } else {
      $archive_cleanup = true
    }

    if ( $zip_extract ) {
      # In $download_dir, will Puppet extract the ZIP files or is this a pre-extracted directory structure.
      if ( $extract_files ) {
        file { "${download_dir}/${file}":
          ensure  => directory,
          owner   => $user,
          group   => $group,
          mode    => '0755',
          require => Db_directory_structure["oracle em structure ${version}"],
        }
      }

      $em_files.each | $filename | {
        if ( $extract_files ) {
          archive { "${download_dir}/${filename}":
            ensure       => present,
            before       => Exec["install oracle em ${title}"],
            cleanup      => $archive_cleanup,
            creates      => "${download_dir}/${file}/installem",
            extract      => $extract_files,
            extract_path => "${download_dir}/${file}",
            group        => $group,
            require      => Db_directory_structure["oracle em structure ${version}"],
            source       => "${mount_point}/${filename}",
            temp_dir     => $temp_dir,
            user         => $user,
          }
        } else {
          archive { "${download_dir}/${filename}":
            ensure   => present,
            before   => Exec["install oracle em ${title}"],
            extract  => $extract_files,
            group    => $group,
            require  => Db_directory_structure["oracle em structure ${version}"],
            source   => "${mount_point}/${filename}",
            temp_dir => $temp_dir,
            user     => $user,
          }
        }
      }
    }

    ora_install::utils::dborainst{"em orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group,
    }

    unless ( defined(File["${download_dir}/em_install_${version}.rsp"]) ) {
      file { "${download_dir}/em_install_${version}.rsp":
        ensure  => present,
        content => template("ora_install/em_install_${version}.rsp.erb"),
        mode    => '0775',
        owner   => $user,
        group   => $group,
        require => [Ora_install::Utils::Dborainst["em orainst ${version}"],
                    Db_directory_structure["oracle em structure ${version}"],],
      }
    }
    unless ( defined(File["${download_dir}/em_install_static_${version}.ini"]) ) {
      file { "${download_dir}/em_install_static_${version}.ini":
        ensure  => present,
        content => template("ora_install/em_install_static_${version}.ini.erb"),
        mode    => '0775',
        owner   => $user,
        group   => $group,
        require => [Ora_install::Utils::Dborainst["em orainst ${version}"],
                    Db_directory_structure["oracle em structure ${version}"],],
      }
    }

    unless ( defined(File[$temp_dir]) ) {
      # Don't touch privs on default OS temp_dir's
      unless ( $temp_dir == '/tmp' or $temp_dir == '/var/tmp' ) {
        file { $temp_dir:
          ensure => present,
          owner  => $user,
          group  => $group,
          mode   => '0755',
          before => Exec["install oracle em ${title}"],
        }
      }
    }

    if ( "${file}.bin" in $em_files ) {
      file { "${download_dir}/${file}.bin":
        ensure => file,
        owner  => $user,
        group  => $group,
        mode   => '0755',
        before => Exec["install oracle em ${title}"]
      }
    }

    exec { "install oracle em ${title}":
      command     => $install_command,
      creates     => $oracle_home_dir,
      environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
      timeout     => 0,
      returns     => [6,0],
      path        => $exec_path,
      cwd         => $oracle_base_dir,
      logoutput   => $logoutput,
      require     => [Ora_install::Utils::Dborainst["em orainst ${version}"],
                      File["${download_dir}/em_install_${version}.rsp"],
                      File["${download_dir}/em_install_static_${version}.ini"],],
    }

    exec { "run root.sh script ${title}":
      command   => $rootsh,
      user      => 'root',
      group     => 'root',
      path      => $exec_path,
      cwd       => $oracle_base_dir,
      logoutput => $logoutput,
      require   => Exec["install oracle em ${title}"],
    }

    file { $oracle_home_dir:
      ensure  => directory,
      recurse => false,
      replace => false,
      mode    => '0775',
      owner   => $user,
      group   => $group,
      require => Exec["install oracle em ${title}","run root.sh script ${title}"],
    }

    # cleanup
    if ( $zip_extract ) {
      exec { 'remove oracle em install files':
        command   => "rm -rf ${download_dir}/${file}",
        user      => $user,
        group     => $group,
        path      => $exec_path,
        logoutput => $logoutput,
        require   => [Exec["install oracle em ${title}"],
                      Exec["run root.sh script ${title}"],],
      }
      if ( $archive_cleanup ) {
        $em_files.each | $filename | {
          file { "${download_dir}/${filename}":
            ensure => absent,
          }
        }
      }
    }
  }
}
