#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: ora_install::opatchupgrade
#
# upgrades oracle opatch
#
#
define ora_install::opatchupgrade(
  Stdlib::Absolutepath                $oracle_home               = undef,
  String[1]                           $patch_file                = undef,
  Optional[Integer]                   $csi_number                = undef,
  Optional[String[1]]                 $support_id                = undef,
  String[1]                           $opversion                 = undef,
  String[1]                           $user                      = 'oracle',
  String[1]                           $group                     = 'dba',
  Stdlib::Absolutepath                $download_dir              = '/install',
  Optional[String[1]]                 $puppet_download_mnt_point = undef,
  Variant[Boolean,Enum['on_failure']] $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
){
  ora_install { $title: ensure => 'present'}

  $exec_path      = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'
  $patch_dir      = "${oracle_home}/OPatch"

  # if a mount was not specified then get the install media from the puppet master
  if ( $puppet_download_mnt_point ) {
    $mount_dir = $puppet_download_mnt_point
  } else {
    $mount_dir = 'puppet:///modules/ora_install'
  }

  # check the opatch version
  $installed_version = opatch_version($oracle_home)
  $version_parts = split($opversion,'\.')
  $major_opatch_version = $version_parts[0]
  #
  # major_opatch_version must be either `11` or `12` at this point in time
  #
  case $major_opatch_version {
    '11','12' : { # Do nothing, this is valid
    }
    default: {
      fail "Major opatch version number ${major_opatch_version} not recognised."
    }
  }

  unless ( $installed_version == $opversion ) {

    case $::kernel {
      'Linux', 'SunOS': {

        notify { "ora_install::opatchupgrade version ${installed_version} installed in ${oracle_home} - upgrade to ${opversion}": }

        archive { "${download_dir}/${title}_${patch_file}":
          ensure       => present,
          cleanup      => true,
          creates      => "${oracle_home}/OPatch/${opversion}",
          extract      => true,
          extract_path => $oracle_home,
          group        => $group,
          source       => "${mount_dir}/${patch_file}",
          user         => $user,
        }

        if ( ($major_opatch_version == '12' and versioncmp($opversion, '12.2.0.1.5') <= 0) or ($major_opatch_version == '11' and versioncmp($opversion, '11.2.0.3.14') <= 0) ) {
          #
          #
          # Older versions need the OCM file. Newer Opatch versions don't support this
          #
          if ( $csi_number and $support_id ) {
            exec { "exec emocmrsp ${title} ${opversion}":
              cwd       => $patch_dir,
              command   => "${patch_dir}/ocm/bin/emocmrsp -repeater NONE ${csi_number} ${support_id}",
              path      => $exec_path,
              user      => $user,
              group     => $group,
              logoutput => $logoutput,
              require   => Archive["${download_dir}/${title}_${patch_file}"],
            }
          } else {

            unless ( defined(Package['expect']) ) {
              package { 'expect':
                ensure  => present,
              }
            }

            file { "${download_dir}/opatch_upgrade_${title}_${opversion}.ksh":
              ensure  => present,
              content => template('ora_install/ocm.rsp.erb'),
              mode    => '0775',
              owner   => $user,
              group   => $group,
            }

            exec { "ksh ${download_dir}/opatch_upgrade_${title}_${opversion}.ksh":
              cwd       => $patch_dir,
              path      => $exec_path,
              user      => $user,
              group     => $group,
              logoutput => $logoutput,
              require   => [
                File["${download_dir}/opatch_upgrade_${title}_${opversion}.ksh"],
                Archive["${download_dir}/${title}_${patch_file}"],
                Package['expect'],
              ],
            }
          }
        }
        file { "${download_dir}/${title}_${patch_file}":
          ensure => absent,
        }
      }
      default: {
        fail('Unrecognized operating system')
      }
    }
  }
}
