#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: ora_install::rcu
#    rcu for soa suite, webcenter
#
#    product = soasuite|webcenter|oim|oam|all
#
#
define ora_install::rcu(
  String[1]                                      $rcu_file                  = undef,
  Enum['soasuite','webcenter','oam','oim','all'] $product                   = 'soasuite',
  String[1]                                      $version                   = '11.1.1.7',
  Optional[Stdlib::Absolutepath]                 $oracle_home               = undef,
  String[1]                                      $user                      = 'oracle',
  String[1]                                      $group                     = 'dba',
  Stdlib::Absolutepath                           $download_dir              = '/install',
  Enum['create','delete']                        $action                    = 'create',
  String[1]                                      $db_server                 = undef,
  String[1]                                      $db_service                = undef,
  String[1]                                      $sys_user                  = 'sys',
  String[1]                                      $sys_password              = undef,
  String[1]                                      $schema_prefix             = undef,
  String[1]                                      $repos_password            = undef,
  Optional[String[1]]                            $temp_tablespace           = undef,
  Optional[String[1]]                            $puppet_download_mnt_point = undef,
  Optional[Boolean]                              $remote_file               = true,
  Variant[Boolean,Enum['on_failure']]            $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
){
  ora_install { $title: ensure => 'present'}

  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  case $::kernel {
    'Linux': {
      $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'
    }
    default: {
      fail('Unrecognized or not supported operating system')
    }
  }

  if ( $puppet_download_mnt_point ) {
    $mount_point = $puppet_download_mnt_point
  } else {
    $mount_point = 'puppet:///modules/ora_install/'
  }

  if ( $mount_point == $download_dir ) {
    $archive_cleanup = false
  } else {
    $archive_cleanup = true
  }


  # create the rcu folder
  unless ( defined(File["${download_dir}/rcu_${version}"]) ) {
    # check rcu install folder
    file { "${download_dir}/rcu_${version}":
      ensure  => directory,
      path    => "${download_dir}/rcu_${version}",
      recurse => false,
      replace => false,
      mode    => '0775',
      owner   => $user,
      group   => $group,
    }
  }

  # download and extract rcu software
  archive { "${download_dir}/${rcu_file}":
    ensure       => present,
    cleanup      => $archive_cleanup,
    creates      => "${download_dir}/rcu_${version}/rcuHome",
    extract      => true,
    extract_path => "${download_dir}/rcu_${version}",
    group        => $group,
    source       => "${mount_point}/${rcu_file}",
    user         => $user,
  }

  unless ( defined(File["${download_dir}/rcu_${version}/rcuHome/rcu/log"]) ) {
    # check rcu log folder
    file { "${download_dir}/rcu_${version}/rcuHome/rcu/log":
      ensure  => directory,
      path    => "${download_dir}/rcu_${version}/rcuHome/rcu/log",
      recurse => false,
      replace => false,
      require => Archive["${download_dir}/${rcu_file}"],
      mode    => '0775',
      owner   => $user,
      group   => $group,
    }
  }

  if $product == 'soasuite' {
    $components           = '-component SOAINFRA -component ORASDPM -component MDS -component OPSS -component BAM'
    $components_passwords  = [$repos_password, $repos_password, $repos_password,$repos_password,$repos_password]
  } elsif $product == 'webcenter' {
    $components           = '-component MDS -component OPSS -component CONTENTSERVER11 -component CONTENTSERVER11SEARCH -component URM -component PORTLET -component WEBCENTER -component ACTIVITIES -component DISCUSSIONS'
    # extra password for DISCUSSIONS and ACTIVITIES
    $components_passwords  = [$repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password]
  } elsif $product == 'oam' {
    $components           = '-component MDS -component OPSS -component IAU -component OAM'
    $components_passwords  = [$repos_password, $repos_password, $repos_password, $repos_password, $repos_password]
  } elsif $product == 'oim' {
    $components           = '-component SOAINFRA -component ORASDPM -component MDS -component OPSS -component BAM -component IAU -component BIPLATFORM -component OIF -component OIM -component OAM -component OAAM -component OMSM'
    $components_passwords  = [$repos_password, $repos_password, $repos_password,$repos_password,$repos_password,$repos_password, $repos_password, $repos_password,$repos_password, $repos_password, $repos_password, $repos_password]
  } elsif $product == 'all' {
    $components           = '-component SOAINFRA -component ORASDPM -component MDS -component OPSS -component BAM -component CONTENTSERVER11 -component CONTENTSERVER11SEARCH -component URM -component PORTLET -component WEBCENTER -component ACTIVITIES -component DISCUSSIONS'
    # extra password for DISCUSSIONS and ACTIVITIES
    $components_passwords  = [ $repos_password, $repos_password, $repos_password,$repos_password,$repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password, $repos_password]
  }

  file { "${download_dir}/rcu_${version}/rcu_passwords_${title}.txt":
    ensure  => present,
    require => Archive["${download_dir}/${rcu_file}"],
    content => template('ora_install/rcu_passwords.txt.erb'),
    mode    => '0775',
    owner   => $user,
    group   => $group,
  }

  if ( $oracle_home ) {
    $pre_command    = "export SQLPLUS_HOME=${oracle_home};${download_dir}/rcu_${version}/rcuHome/bin/rcu -silent"
  } else {
    $pre_command    = "${download_dir}/rcu_${version}/rcuHome/bin/rcu -silent"
  }
  $post_command     = "-databaseType ORACLE -connectString ${db_server}:${db_service} -dbUser ${sys_user} -dbRole SYSDBA -schemaPrefix ${schema_prefix} ${components} "
  $password_command = " -f < ${download_dir}/rcu_${version}/rcu_passwords_${title}.txt"

  #optional set the Temp tablespace
  if ( $temp_tablespace ) {
    $create_command  = "${pre_command} -createRepository ${post_command} -tempTablespace ${temp_tablespace} ${password_command}"
  } else {
    $create_command  = "${pre_command} -createRepository ${post_command} ${password_command}"
  }
  $delete_command  = "${pre_command} -dropRepository ${post_command} ${password_command}"

  if $action == 'create' {
    $statement = $create_command
  }
  elsif $action == 'delete' {
    $statement = $delete_command
  }

  db_rcu { $schema_prefix:
    ensure       => $action,
    statement    => $statement,
    os_user      => $user,
    oracle_home  => $oracle_home,
    sys_user     => $sys_user,
    sys_password => $sys_password,
    db_server    => $db_server,
    db_service   => $db_service,
    require      => [Archive["${download_dir}/${rcu_file}"],
                    File["${download_dir}/rcu_${version}/rcu_passwords_${title}.txt"],],
  }

}
